//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef EMBEDDERMDS_H
#define EMBEDDERMDS_H

class Sequence;
class MovableEmbedderEllipse;
#include <Eigen/Core>
#include <QObject>
#include <QRectF>
#include <vector>

class EmbedderMDS : public QObject
{
    Q_OBJECT

public:
    EmbedderMDS(const Eigen::MatrixXd &normalizedFeatures, Sequence &sequence);
    double frameX(int frame) const;
    double frameY(int frame) const;
    QRectF boundingRect() const;
    void setPoint(int frame, MovableEmbedderEllipse *point);
    void unpin(int frame);
    void unpinSelected();
    void unpinAll();
    void pointUpdated(MovableEmbedderEllipse *point);
    void iterateToConvergence();
    bool iteratingNow() const;

signals:
    void embeddingChanged();

private:
    void syncWithPoints();
    void calcRealDist(int i, int j);
    double embedDist(int i, int j) const;
    double stress() const;

    Eigen::MatrixXd realDists;
    const Eigen::MatrixXd &normalizedFeatures;
    bool iterating;
    Sequence &sequence;
    Eigen::MatrixXd embedding;
    std::vector<MovableEmbedderEllipse *> points;
};

#endif // EMBEDDERMDS_H
