//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LINROTINVREP_H
#define LINROTINVREP_H

#include "Rep.h"
#include <Eigen/Core>
#include "openmesh.h"

class LinRotInvRep : public Rep {
    friend class LinRotInvRepConverter;

public:
    LinRotInvRep() {}
    LinRotInvRep(const LinRotInvRepConverter &converter, Mesh &mesh);

    Eigen::VectorXd getVector() const;
    void setFromVector(const Eigen::VectorXd &vec);
    Description getDescription() const;

    LinRotInvRep operator+(const LinRotInvRep &other) const;
    LinRotInvRep operator-(const LinRotInvRep &other) const;
    LinRotInvRep operator*(double mult) const;
    LinRotInvRep &operator+=(const LinRotInvRep &other);
    LinRotInvRep &operator-=(const LinRotInvRep &other);
    LinRotInvRep &operator*=(double mult);

private:
    Eigen::VectorXd edgeLengths;
    Eigen::VectorXd edgeAngles;
    Eigen::VectorXd secondForms;
    Eigen::Vector3d barycenter;
    Eigen::Vector3d firstFrame;
};

inline LinRotInvRep operator*(double mult, const LinRotInvRep &other)
{
    return other * mult;
}

#endif // LINROTINVREP_H
