//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef REPCONVERTER_H
#define REPCONVERTER_H

#include "openmesh.h"
#include <Eigen/Core>

class RepConverter {
protected:
    static const Eigen::Matrix3d expM(const Eigen::Vector3d &v);
    static const Eigen::Vector3d logM(const Eigen::Matrix3d &R);
    static const Eigen::Vector3d sqrtMatrix(const Eigen::Matrix3d &R);
};

template <typename Scalar, int N>
inline Eigen::Matrix<Scalar, N, 1> fromOMVec(const OpenMesh::VectorT<Scalar, N> &vec)
{
    return Eigen::Map< const Eigen::Matrix<Scalar, N, 1> >(vec.data());
}

template <typename Scalar, int N>
inline OpenMesh::VectorT<Scalar, N> fromEigenVec(const Eigen::Matrix<Scalar, N, 1> &vec)
{
    OpenMesh::VectorT<Scalar, N> ret;
    for (int i = 0; i < N; ++i) ret[i] = vec[i];
    return ret;
}

#endif // REPCONVERTER_H
