//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#include "RepConverter.h"

#include <Eigen/Geometry>

const Eigen::Matrix3d RepConverter::expM(const Eigen::Vector3d &v)
{
    double hSinchTh, theta = v.norm();
    if (theta < 1e-4) {
        hSinchTh = 0.5 + theta * theta / 48.0;
    } else {
        hSinchTh = sin(0.5 * theta) / theta;
    }

    Eigen::Quaternion<double> q(cos(0.5 * theta),
                                hSinchTh * v(0),
                                hSinchTh * v(1),
                                hSinchTh * v(2));
    return q.toRotationMatrix();
}

const Eigen::Vector3d RepConverter::logM(const Eigen::Matrix3d &R)
{
    double angle = acos((R.trace() - 1) * 0.5);

    if (!_finite(angle)) {
        angle = 0;
    }
    if (!angle) return Eigen::Vector3d(0, 0, 0);

    if (angle == M_PI)
    {
        return sqrtMatrix((R - Eigen::Matrix3d::Identity()) * 0.5) * angle;
    }

    Eigen::Matrix3d B = (0.5 * (R - R.transpose())) / sin(angle);
    return Eigen::Vector3d(B(2, 1), B(0, 2), B(1, 0)) * angle;
}

const Eigen::Vector3d RepConverter::sqrtMatrix(const Eigen::Matrix3d &R)
{
    double a = R(0, 0) - R(1, 1) - R(2, 2);
    a = a < 0 ? 0 : sqrt(0.5 * a);

    double b = -R(0, 0) + R(1, 1) - R(2, 2);
    b = b < 0 ? 0 : sqrt(0.5 * b);

    double c = -R(0, 0) - R(1, 1) + R(2, 2);
    c = c < 0 ? 0 : sqrt(0.5 * c);

    return Eigen::Vector3d(a, b, c);
}
