//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#include "VertexRepConverter.h"

void VertexRepConverter::setMeshFromRep(Mesh &targetMesh, const VertexRep &rep) const
{
    int i = 0;
    for (Mesh::VertexIter v_it = targetMesh.vertices_begin();
        v_it != targetMesh.vertices_end(); ++v_it, ++i)
    {
        Mesh::Point p(static_cast<float>(rep.coords[3 * i]),
                      static_cast<float>(rep.coords[3 * i + 1]),
                      static_cast<float>(rep.coords[3 * i + 2]));
        targetMesh.set_point(v_it, p);
    }
}

void VertexRepConverter::setRepFromMesh(VertexRep &rep, const Mesh &targetMesh) const
{
    rep.coords = Eigen::VectorXd::Zero(3 * targetMesh.n_vertices());

    int i = 0;
    for (Mesh::ConstVertexIter v_it = targetMesh.vertices_begin();
        v_it != targetMesh.vertices_end(); ++v_it, ++i)
    {
        Mesh::Point p = targetMesh.point(v_it);
        rep.coords[3 * i    ] = p[0];
        rep.coords[3 * i + 1] = p[1];
        rep.coords[3 * i + 2] = p[2];
    }
}
